#include "AlarmingWidget.h"
#include "AlarmingTheme.h"
#include <QApplication>
#include <QPainter>
#include <QFont>
#include <QColor>
#include <QFontMetrics>
#include <QPixmap>
#include <QPushButton>
#include <QTimer>
#include <QMouseEvent>
#include <QBitmap>

AlarmingWidget::AlarmingWidget(
   AlarmingTheme * theme,
   QStringList reminder,
   int init_snooze_ms,
   const char * name)
 : QWidget(0, AlarmingWidget::WindowFlags(theme)),
   snooze_ms(init_snooze_ms),
   snooze_timer(NULL) {

   setAccessibleName(name);

   // Get the background pixmap and reminder text setup

   DrawReminderText(theme, reminder);

   // Then do the buttons.

   QIcon * ok_icons = theme->OKButtonIcons();
   if (ok_icons == NULL) {
      ok_btn = new QPushButton(theme->OKButtonText(), this);
   } else {
      ok_btn = new QPushButton(*ok_icons, theme->OKButtonText(), this);
   }
   QRect ok_geo = theme->OKButtonPosition();
   ok_btn->setGeometry(ok_geo.x(), ok_geo.y(), ok_geo.width(), ok_geo.height());
   QObject::connect(ok_btn, SIGNAL(clicked()), qApp, SLOT(quit()));

   QIcon * snooze_icons = theme->SnoozeButtonIcons();
   if (snooze_icons == NULL) {
      snooze_btn = new QPushButton(theme->SnoozeButtonText(), this);
   } else {
      snooze_btn = new QPushButton(*snooze_icons, theme->SnoozeButtonText(),
                                   this);
   }
   QRect snooze_geo = theme->SnoozeButtonPosition();
   snooze_btn->setGeometry(snooze_geo.x(), snooze_geo.y(),
                           snooze_geo.width(), snooze_geo.height());
   connect(snooze_btn, SIGNAL(clicked()), SLOT(doSnooze()));
}

AlarmingWidget::~AlarmingWidget() {
   delete ok_btn;
   delete snooze_btn;
   delete snooze_timer;
}

void
AlarmingWidget::mousePressEvent(QMouseEvent * e) {
   clickPos = e->pos() + QPoint(geometry().topLeft() -
                                frameGeometry().topLeft());
}

void
AlarmingWidget::mouseMoveEvent(QMouseEvent * e) {
   move(e->globalPos() - clickPos);
}

Qt::WindowFlags
AlarmingWidget::WindowFlags(AlarmingTheme * theme) {
   unsigned long rval = (unsigned long)Qt::Window;
   rval |= (unsigned long)Qt::CustomizeWindowHint;
   rval |= (unsigned long)Qt::FramelessWindowHint;
   if (theme->ShouldStayOnTop()) {
      rval |= (unsigned long)Qt::WindowStaysOnTopHint;
   }
   if (theme->ShouldBypassX11WM()) {
      rval |= (unsigned long)Qt::X11BypassWindowManagerHint;
   }
   return (Qt::WindowFlags)rval;
}

static QRect
tot_text_rect(QStringList reminder, QFont * msg_font)
{
   QRect rval(0,0,0,0);
   QFontMetrics qfm(*msg_font);
   for (QStringList::Iterator it = reminder.begin();
        it != reminder.end();
        ++it) {
      QRect line_i = qfm.boundingRect(*it);
      if (line_i.width() > rval.width()) {
         rval.setWidth(line_i.width());
      }
      rval.setHeight(rval.height() + qfm.lineSpacing());
   }
   return rval;
}

void
AlarmingWidget::DrawReminderText(
   AlarmingTheme * theme,
   QStringList reminder) {

   // Tricky stuff to fit the message into the text rectangle in the
   // largest size font that will fit. Want to paint this into the
   // Pixmap before we set it as the widget background.

   // FIXME - I bet this really needs to be even more tricky for total
   // internationalization, not to mention adding left/center/right and
   // top/center/bottom customizations to the text position...
   // Probably if I knew enough there is some text render engine
   // I could use in Qt and not do all this work myself.

   // Use the theme's font as a starting place.

   QFont * msg_font = theme->ReminderTextFont();
   QRect text_box = theme->ReminderTextBox();
   int text_width = text_box.width();
   int text_height = text_box.height();

   // First try to scale the font at one swell foop to be the right size,
   // then tweak it if it is still too big.

   QRect r = tot_text_rect(reminder, msg_font);
   float cur_point = msg_font->pointSizeF();
   float width_point = cur_point;
   float height_point = cur_point;
   width_point = (width_point * (float)text_width) / (float)r.width();
   height_point = (height_point * (float)text_height) / (float)r.height();
   if (width_point < height_point) {
      height_point = width_point;
   }
   msg_font->setPointSizeF(height_point);
   cur_point = msg_font->pointSizeF();

   // Font we have now has likely been rounded to be too big. Reduce it in
   // size 10% at a time until everything fits or we get tired of trying to
   // make it fit.

   for (int tries = 0; tries < 5; ++tries) {
      r = tot_text_rect(reminder, msg_font);
      if ((r.width() <= text_width) && (r.height() <= text_height)) {
         break;
      }
      cur_point = cur_point * 0.9;
      msg_font->setPointSizeF(cur_point);
   }

   // Now draw the message centered in the text area

   int x = text_box.x();
   int y = text_box.y() + ((text_height - r.height())/2);
   QFontMetrics qfm(*msg_font);
   QPainter alarm_painter(theme->AlarmBackground());
   alarm_painter.setFont(*msg_font);
   alarm_painter.setPen(*theme->ReminderTextColor());
   for (QStringList::Iterator it = reminder.begin();
        it != reminder.end();
        ++it) {
      QRect line_i = qfm.boundingRect(*it);
      alarm_painter.drawText(x + (text_width - line_i.width())/2,
                             y + qfm.ascent(), *it);
      y += qfm.lineSpacing();
   }

   resize(theme->AlarmBackground()->width(),
          theme->AlarmBackground()->height());
   QPalette palette;
   palette.setBrush(backgroundRole(), QBrush(*theme->AlarmBackground()));
   setPalette(palette);
   setMask(theme->AlarmBackground()->mask());
}

void
AlarmingWidget::doSnooze() {
   if (snooze_timer == NULL) {
      snooze_timer = new QTimer(this);
      connect(snooze_timer, SIGNAL(timeout()), SLOT(wakeUp()));
   }
   snooze_timer->start(snooze_ms);
   hide();
}

void
AlarmingWidget::wakeUp() {
   show();
}
